var Control = ol.control.Control;
var Draw = /*@__PURE__*/(function (Control) {
  function DrawControl(opt_options) {
    console.log("Draw Control");
    var settings = opt_options || {};
    
    settings.format = new ol.format.WKT();
    var geomType = null;
    var geom = null;
    var mapid = settings.mapid;
        
    settings.wkt = this.readWKT(mapid, settings.parameter.target);
    if (settings.wkt) {
      geomType = this.getGeomType(settings.wkt);
      settings.geom = this.createGeomFromWKT(settings.wkt, settings.format);
    }
    
    var types = settings.parameter.type.split("|");
    var tooltips = settings.tooltip.split("|");
    var icons = settings.icon.split("|");
    var groupelement = document.createElement('div');
    groupelement.className = 'ol-control-btn-group';

    for (var i = 0; i< types.length; i++) {
      var element = document.createElement('div');
      element.className = 'ol-unselectable ol-control';
      var button = document.createElement('img');
      button.className = "ol-control-icon ";
      var protocol = location.protocol;
      var slashes = protocol.concat("//");
      var host = slashes.concat(window.location.hostname);
      button.src = host + settings.path +"/img/" + icons[i];
      button.title = tooltips[i];
      if(types[i] === geomType) {
        button.className = "ol-control-icon" + " " + 'activated';
      }
      button.setAttribute('geometry', types[i]);    
      button.addEventListener('click', this.handleDraw.bind(this, settings), false);
      element.appendChild(button);
      groupelement.appendChild(element);
    }
    window.addEventListener('DOMContentLoaded', this.drawInit.bind(this, settings), false);
    
    Control.call(this, {
      element: groupelement,
      target: settings.target
    });
  }

  if ( Control ) DrawControl.__proto__ = Control;
  DrawControl.prototype = Object.create( Control && Control.prototype );
  DrawControl.prototype.constructor = DrawControl;

  DrawControl.prototype.handleDraw = function handleDraw (settings,event) {
    var element = event.target;
    if(element.classList.contains('activated')) {
      element.classList.remove('activated');
      this.removeDrawControl();
    } else {
      var parent = element.parentElement.parentElement;
      var activeChild = parent.getElementsByClassName('activated');
      if(activeChild.length > 0 ) {
        activeChild[0].classList.remove('activated');
        this.removeDrawControl();
      }
      element.classList.add('activated');
      this.addDrawControl(element.getAttribute('geometry'), settings.mapid, settings.parameter.target, settings.format);
    }
  }
  
  DrawControl.prototype.addDrawControl = function addDrawControl(geomType, mapid, target, format) {
    console.log('addDrawControl: ' + geomType);
    var layers = [];
    this.getMap().getLayers().forEach(function (layer) {
      if (layer.get('name') !== undefined && layer.get('name') === 'drawLayer') {
        layers.push(layer);
      }
    });
    var inputSource = layers[0].getSource();
    var draw = new ol.interaction.Draw({
          source: inputSource,
          type: /** @type {ol.geom.GeometryType} */ geomType,
    });
    draw.set('name','drupalDraw');
    
    draw.on('drawstart', function(evt) {
      if(!geomType.includes("Multi")) {
        target = target.replace('{uuid}',mapid);
        target = target.replace (/(\r\n|\n|\r)/gm, "");
    
        if(target && document.getElementById(target)) {
          document.getElementById(target).value = '';
        }
        inputSource.clear();
      }
    });
        
    draw.on('drawend', function(evt) {
      console.log('drawEnd')  
      
      target = target.replace('{uuid}',mapid);
      target = target.replace (/(\r\n|\n|\r)/gm, "");
      
      var features = inputSource.getFeatures();
      if (geomType.includes("Multi")) {
        var geom = null;
        switch (geomType) {
          case 'MultiPoint':
            console.log('Erzeuge MultiPoint');
            geom = new ol.geom.MultiPoint();
            break;
          case 'MultiLineString':
            console.log('Erzeuge MultiLineString');
            geom = new ol.geom.MultiLineString();
            break;
          case 'MultiPolygon':
            console.log('Erzeuge MultiPolygon');
            geom = new ol.geom.MultiPolygon();
            break;
        }
        
        var feature = evt.feature;
        switch (geomType) {
          case 'MultiPoint':
            geom.appendPoint(feature.getGeometry());
            break;
          case 'MultiLineString':
            geom.appendLineString(feature.getGeometry());
            break;
          case 'MultiPolygon':
            geom.appendPolygon(feature.getGeometry());
            break;
        }
            
        if (features.length > 0) {
          features.forEach(function(item) {
            switch (geomType) {
              case 'MultiPoint':
                geom.appendPoint(item.getGeometry());
                break;
              case 'MultiLineString':
                geom.appendLineString(item.getGeometry());
                break;
              case 'MultiPolygon':
                geom.appendPolygon(item.getGeometry());
                break;
            }
          });
        }       
        feature.setGeometry(geom);
            
        document.getElementById(target).value = format.writeFeatures(features, {
          dataProjection: 'EPSG:4326',
          featureProjection: 'EPSG:3857'
        });
      } else {
        features.push(evt.feature);
        if(target && document.getElementById(target)) {
          document.getElementById(target).value = format.writeFeatures(features, {
            dataProjection: 'EPSG:4326',
            featureProjection: 'EPSG:3857'
          });
        }
      }      
    });
    
    var modify = new ol.interaction.Modify({
      source: inputSource,
      name: 'drawModify'
    });
    var snap = new ol.interaction.Snap({
      source: inputSource,
      name: 'drawSnap'
    });
    this.getMap().addInteraction(snap);
    this.getMap().addInteraction(modify);
    this.getMap().addInteraction(draw);
  }
  
  DrawControl.prototype.removeDrawControl = function removeDrawControl() {
    console.log('removeDrawControl');
    this.getMap().getInteractions().forEach(function (interaction, map) {
      if(interaction.get('name') === 'drupalDraw' || interaction.get('name') === 'drupalModify' || interaction.get('name') === 'drupalSnap') {
        interaction.setActive(false);
      }
    });
  }
  
  DrawControl.prototype.readWKT = function readWKT (mapid, target) {
    var wkt = null;
    target = target.replace('{uuid}',mapid);
    target = target.replace (/(\r\n|\n|\r)/gm, "");
    
    if(target && document.getElementById(target)) {
      wkt = document.getElementById(target).value;
    }
    
    return wkt;
  }
  
  DrawControl.prototype.drawInit = function drawInit(settings) {
    var inputSource = new ol.source.Vector({wrapX: false});
    var inputLayer = new ol.layer.Vector({
      source: inputSource,
      name: 'drawLayer'
    }); 
    this.getMap().addLayer(inputLayer);
    if(settings.geom) {
      inputSource.addFeature(settings.geom);
      const extent = settings.geom.getGeometry().getExtent()
      this.getMap().getView().fit(extent);
    }
    this.addDrawControl(settings.geom.getGeometry().getType(), settings.mapid, settings.parameter.target, settings.format);
  }
    
  DrawControl.prototype.getGeomType = function getGeomType(wkt) {
    var geomType = wkt.replace(/[ (),-.0-9]/g,"");
    geomType = geomType.toUpperCase();
    if (geomType === "LINESTRING") {
      geomType = 'LineString';
    }
    if (geomType === "POLYGON") {
      geomType = 'Polygon';
    }
    if (geomType === "POINT") {
      geomType = 'Point';
    }
    if (geomType === "MULTIPOINT") {
      geomType = 'MultiPoint';
    }
    if (geomType === "MULTILINSESTRING") {
      geomType = 'MultiLinseString';
    }
    if (geomType === "MULTIPOLYGON") {
      geomType = 'MultiPolygon';
    }
    if (geomType === "GEOMETRYCOLLECTION") {
      geomType = 'GeometryCollection';
    }
    return geomType;
  }
  
  DrawControl.prototype.createGeomFromWKT = function createGeomFromWKT(wkt, format) {
    try {    
      var feature = format.readFeature(wkt, {
        dataProjection: 'EPSG:4326',
        featureProjection: 'EPSG:3857'
      });
      return feature;
    } catch(e) {
      console.log("Error:");
      console.log(e)
    }
    return null;
   }
  
  return DrawControl;
}(Control));